<?php
namespace Teknisa\DF\Modules\Entrada\Controllers;

/* Bibliotecas */
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Row;

use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\FileUploader\Server as fileUploader;

/* Classes DF */
use Teknisa\DF\Controllers\TeknisaCrud;
use Teknisa\DF\Modules\Entrada\Services\Dfe71000SincronizacaoCTEService;
use Teknisa\DF\Modules\Entrada\Services\ImportacaoConsultaNotaFiscalService;
use Teknisa\DF\Modules\Services\Ger40100IntegracaoViaEmailService;

class ImportacaoConsultaNotaFiscalController extends TeknisaCrud {
    
    protected $dataSourceName       = 'intgnotafiscal_fornecedor';
    protected $parameterBagColumns  = array("P_CDPAIS", "P_SGESTADO", "P_CDFILIAL", "P_CDFORNECED", "P_DTINICIAL", "P_DTFINAL", "P_DATA", "P_IDSTATUSNFE", "P_CDINSCESTA", "P_NRNOTAFISCINI", "P_NRNOTAFISCFIN", "P_NRNOTA", "P_NRACESSONFE", "P_IDDOCFISCELE");
    protected $connectionCustom;
    protected $dfe51000ImportacaoConsultaNotaFiscalService;
    protected $sincronizacaoCTEService;
    protected $ger40100IntegracaoViaEmailService;
    
    const PRODUCTION_SERVER_URL = 'https://midia.teknisa.com/index.php';
    
    public function __construct($crudService, ParameterBag $parameterBag, $dfe51000ImportacaoConsultaNotaFiscalService, $importacaoNFSeService, $birtController, $connectionCustom, $fileUploader,
                                $sincronizacaoCTEService, $ger40100IntegracaoViaEmailService) {
                                    
        $this->dfe51000ImportacaoConsultaNotaFiscalService = $dfe51000ImportacaoConsultaNotaFiscalService;
        $this->birtController   = $birtController;
        $this->connectionCustom = $connectionCustom;
        parent::__construct($crudService, $parameterBag);
        $this->importacaoNFSeService = $importacaoNFSeService;
        $this->fileUploader  = $fileUploader;
        $this->sincronizacaoCTEService = $sincronizacaoCTEService;
        $this->ger40100IntegracaoViaEmailService = $ger40100IntegracaoViaEmailService;
        set_time_limit( 0 ); // Limite de tempo de execução: 2h. Deixe 0 (zero) para sem limite
        ignore_user_abort( true );
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        parent::find($request, $response);
    }
    
    public function updateNumeroAfXml(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrAcessoNfe = $row["NRACESSONFE"];
        $nrAutoForn  = $row["NRAUTOFORN"];
        $nrSeqItProd = (isset($row["NRSEQITPROD"])) ? $row["NRSEQITPROD"] : '';
        
        $this->dfe51000ImportacaoConsultaNotaFiscalService->updateNumeroAfXml($nrAcessoNfe, $nrAutoForn, $nrSeqItProd);
    }
    
    public function gerarNfXml(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $nrAcessoNfe = $row["NRACESSONFE"];
        $cdTipoOper = $row["CDTIPOOPER"];
        
        $nf = $this->dfe51000ImportacaoConsultaNotaFiscalService->gerarNfXml($nrAcessoNfe, $cdTipoOper);
        
        $response->addDataSet(new DataSet("gerarNfXml", $nf));
    }
    
    public function statusNfeXml(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $nrAcessoNfe = $row["NRACESSONFE"];
        $idStatusNfe = $row["IDSTATUSNFE"];
        $cdOperador = $row["CDOPERADOR"];
        $dtHistorico = $row["DTHISTORICO"];
        $idStatus = $row["IDSTATUS"];
        $nrSeqHistor = $this->dfe51000ImportacaoConsultaNotaFiscalService->getSequencialHistoricoXML($nrAcessoNfe);
        $params = array(

            "NRACESSONFE"   =>  $nrAcessoNfe,
            "NRSEQHISTOR"   =>  $nrSeqHistor,
            "DTHISTORICO"   =>  $dtHistorico,
            "IDTIPOOPER"    =>  '2',
            "IDSTATUS"      =>  $idStatus,
            "NRPROTOCOLONFE"=>  ' ',
            "DSOPERACAO"    =>  ' ',
            "CDFILIAL"      =>  ' ',
            "NRLANCTONF"    =>  ' ',
            "CDOPERADOR"    =>  $cdOperador
        );


        if($idStatusNfe == 'N' && $idStatus == 'N' ){
            $mensagem = 'Bloqueada';
            $params["IDSTATUS"] =  'B';
            $idStatusNfe = 'B';
 
        }
        else {
            $mensagem = 'Desbloqueada';
            $params["IDSTATUS"] =  'N';
            $idStatusNfe = 'N';
        } 

        $params["DSOPERACAO"] = $mensagem;

        $this->dfe51000ImportacaoConsultaNotaFiscalService->insereXml($params);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->atualizaStatusXml($nrAcessoNfe, $idStatusNfe);

 
    }
    
    public function getPainelInconsistencias(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "NRACESSONFE") $nrAcessoNfe = $condition["value"];
        }
        
        if(!$nrAcessoNfe) $response->addDataSet(new DataSet("consistxml_itpanconsxml", array()));
        else {
            $response->addDataSet(new DataSet("consistxml_itpanconsxml", $this->dfe51000ImportacaoConsultaNotaFiscalService->getPainelInconsistencias($nrAcessoNfe)));
        }
    }
     
    public function getItensInconsistencias(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "NRACESSONFE") $nrAcessoNfe   = $condition["value"];
            else if($condition["columnName"] == "CDPAINEL") $cdPainel = $condition["value"];
        }
        
        if(!$nrAcessoNfe || !$cdPainel) $response->addDataSet(new DataSet("panconsxml_itpanconsxml", array()));
        else {
            $response->addDataSet(new DataSet("panconsxml_itpanconsxml", $this->dfe51000ImportacaoConsultaNotaFiscalService->getItensInconsistencias($nrAcessoNfe, $cdPainel)));
        }
        
    }
    
    public function gerarDanfe(DTO\Request\Filter $request, DTO\Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $row = $requestConditions[0]["value"];
        $nracessonfe = $row["NRACESSONFE"];

        try {
            $xml = $this->dfe51000ImportacaoConsultaNotaFiscalService->getXml($nracessonfe);
            $danfe = $this->dfe51000ImportacaoConsultaNotaFiscalService->gerarDanfe($xml, $nracessonfe);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('danfe', array($danfe) ));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function geraRelatorioImpCons(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $rowBirt = new Row();
        
        $rowBirt->set("__report_name", $row["IDREPORT"]);
        $rowBirt->set("P_NRACESSONFE", (isset($row["NRACESSONFE"])) ?  ($row["NRACESSONFE"] === "" ? 'T' : $row["NRACESSONFE"] )  : 'T');
        
        if($row["IDREPORT"] !== 'DFE51002') {
            $rowBirt->set("P_NRACESSONFE", (isset($row["NRACESSONFE"])) ? ($row["NRACESSONFE"] === "" ? 'T' : $row["NRACESSONFE"] )  : 'T');
            $rowBirt->set("P_CDFORNECED",  (isset($row["CDFORNECED"]))  ? ($row["CDFORNECED"] === "" ? 'T' : $row["CDFORNECED"] )   : 'T');
            $rowBirt->set("P_CDSERIE",     (isset($row["CDSERIE"]))     ? ($row["CDSERIE"] === "" ? 'T' : $row["CDSERIE"] )      : 'T');
            $rowBirt->set("P_NRNOTAFISC",  (isset($row["NRNOTAFISC"]))  ? ($row["NRNOTAFISC"] === "" ? 'T' : $row["NRNOTAFISC"] )    : 'T');
        }
        else {
             $rowBirt->set("P_DSINCONSISTENCIA", (isset($row["DSINCONSISTENCIA"])) ?  ($row["DSINCONSISTENCIA"] === "" ? 'T' : $row["DSINCONSISTENCIA"] ): 'T');
        }
        
        if($row["IDREPORT"] != 'DFE51000') {
            $rowBirt->set("P_NMARQUIVO", (isset($row["NMARQUIVO"])) ? ($row["NMARQUIVO"] === "" ? 'T' : $row["NMARQUIVO"] )   : 'T');
            $rowBirt->set("P_DSINCONSISTENCIA", (isset($row["DSINCONSISTENCIA"])) ? ($row["DSINCONSISTENCIA"] === "" ? 'T' : $row["DSINCONSISTENCIA"] ):'T');
        }
        
        $rowBirt->set("P_NRORG", $row["NRORG"] );
        $requestBirt = new Request\Row($rowBirt, "POST", "/createRemoteReport", $request->getUserId(), 'aaa');
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        // $response->addDataSet(new DataSet("panconsxml_itpanconsxml", $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getItensInconsistencias($nrAcessoNfe, $cdPainel)));
    }
    
    public function importarNotasFiscais(DTO\Request\Row $request, DTO\Response $response) {
        $this->dfe51000ImportacaoConsultaNotaFiscalService->limpaLoginXML();
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarNotas();
        
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
            
            $emailPendente = $this->dfe51000ImportacaoConsultaNotaFiscalService->enviaEmailRelacionamentoPendente();
            $response->addMessage(new Response\Message($emailPendente));
            //$this->dfe51000ImportacaoConsultaNotaFiscalService->callEnviarEmailNfPendente();
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
 
        if(count($erros) > 0) {
             $response->addNotification(new Response\Notification($retorno["ERROS"]));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }
    }
    
    public function uploadNotasFiscais(DTO\Request\Row $request, DTO\Response $response) {
        $rows = $request->getRow();
        $useVpd = InstanceManager::getInstance()->getService('EntityManager')->getConnection()->getParams()['useVpd'];
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->uploadNotas($rows, $useVpd);
        
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();

        if(count($erros) > 0) {
             $response->setError(new Response\Error($retorno["ERROS"], 500));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }
    }
    
    public function importarFacturacionElectronica(DTO\Request\Row $request, DTO\Response $response) {
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarFacturacionElectronica();
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();

        if(count($erros) > 0) {
             $response->addNotification(new Response\Notification($retorno["ERROS"]));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }

    }
    
    public function importarCTe(DTO\Request $request, DTO\Response $response) {
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarCTe();
        if($retorno["ERROS"] == 0) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]." CTe importados com sucesso."));
        }
        else {
            if($retorno["SUCESSOS"] > 0 ) {
                $response->addMessage(new Response\Message($retorno["SUCESSOS"]." CTe importados com sucesso.<br>".$retorno["ERROS"]." CTe com erro. Verifique as inconsistências."));
            }
            else {
                $response->addMessage(new Response\Message($retorno["ERROS"]." CTe com erro. Verifique as inconsistências."));
            }
            $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
            foreach ($erros as $erro) {
                $response->addMessage(new Response\Message($erro));
            }
        }
    }
    
    public function getDefaultFilterImportacao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $defaultFilter = $this->dfe51000ImportacaoConsultaNotaFiscalService->getDefaultFilterImportacao($row);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultFilter', $defaultFilter));
           
    }
    
    public function enviaEmailRelacionamentoPendente(DTO\Request $request, DTO\Response $response) {
        $emailPendente = $this->dfe51000ImportacaoConsultaNotaFiscalService->enviaEmailRelacionamentoPendente();
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('enviaEmailRelacionamentoPendente', $emailPendente));
        
        /*if(isset($row["VPD"]) && isset($row["NRORG"])) {
             $entityManager = $this->connectionCustom->updateVpdConnection($row["NRORG"]);
        }*/
    }
     
    public function getFilialCNPJ (DTO\Request\Filter $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceName;
        
        $this->dataSourceName = "filial";
        parent::find($request, $response);
        $rows = $response->getDataSets()[0]->getRows();
        
        foreach($rows as &$filial){
            $this->dfe51000ImportacaoConsultaNotaFiscalService->getFilial($filial['CDEMPRESA'], $filial['NRINSJURFILI']);
        }
        
        $dataSet = new \Zeedhi\Framework\DataSource\DataSet('filial', $rows);
        $response->setDataSets(array($dataSet));
  
    }
    
    public function getInscEstadual(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $cdfilial = $row['CDFILIAL'];
        $inscestad = $this->dfe51000ImportacaoConsultaNotaFiscalService-> getInscEstadual($cdfilial);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getInscEstadual', $inscestad));
    }
       
    public function getRepresentante (DTO\Request\Filter $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceName;
        $this->dataSourceName = "inscrestad";
        parent::find($request, $response);
        $rows = $response->getDataSets()[0]->getRows();
        foreach($rows as &$inscricao){
            if(!$inscricao['NMREPRINSEST'])
                $inscricao['NMREPRINSEST'] = $inscricao['CDINSCESTA'];
        }
        
        $dataSet = new \Zeedhi\Framework\DataSource\DataSet('inscrestad', $rows);
        $response->setDataSets(array($dataSet));
        
    }

    public function importarNFSe(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $retorno = $this->importacaoNFSeService->sincronizarNFSe($row);
        
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
            //$this->dfe51000ImportacaoConsultaNotaFiscalService->callEnviarEmailNfPendente();
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
 
        if(count($erros) > 0) {
             $response->addNotification(new Response\Notification($retorno["ERROS"]));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }
    }
    
    public function updateDesvinculaAfProduto(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrAcessoNfe = $row["NRACESSONFE"];
        $nrSeqItProd = $row["NRSEQITPROD"];
        $nrAutoForn  = null;
        
        $this->dfe51000ImportacaoConsultaNotaFiscalService-> updateDesvinculaAfProduto($nrAcessoNfe, $nrAutoForn, $nrSeqItProd);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->desvinculaAfProduto($nrAcessoNfe, $nrSeqItProd);

    }
    
    public function updateDesvinculaAfNotafiscal(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrAcessoNfe = $row["NRACESSONFE"];
        $nrAutoForn  =  null;

        $this->dfe51000ImportacaoConsultaNotaFiscalService-> updateDesvinculaAfNotafiscal($nrAcessoNfe, $nrAutoForn);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->desvinculaAfNotafiscal($nrAcessoNfe);
    }
    
    public function agendamentoManifestacaoImportacao(Request $request, Response $response) {
        var_dump('O agendamento sera iniciado.');
        $row = $request->getParameters();
        
        if(empty($row['NRORG'])) throw new \Exception('Parametro obrigatorio NRORG nao esta informado corretamente.');
        
        $nrorg = $row['NRORG'];
        $_SESSION['_sf2_attributes']['NRORG']           = $nrorg;
        $_SESSION['_sf2_attributes']['ORGANIZATION_ID'] = $nrorg;
        $newEntityManager = null;
        $useVpd = InstanceManager::getInstance()->getService('EntityManager')->getConnection()->getParams()['useVpd'];
        
        
        //Configura o agendamento.
        $this->setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager);
        
        //Configura a manifestacao NFe e importacao NFe/CTe.
        $this->dfe51000ImportacaoConsultaNotaFiscalService->getManifestacaoService()->setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $this->fileUploader);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->setAgendamentoImportacao($nrorg, $useVpd, $newEntityManager, $this->fileUploader);
        
        //Inicio integração por email, importacao NFe por meio dos emails cadastrados.
        $this->ger40100IntegracaoViaEmailService->setAgendamentoIntegracaoEmail($nrorg, $useVpd, $newEntityManager, $this->fileUploader);
        $this->ger40100IntegracaoViaEmailService->verificaTodosEmails($row);
        
        $empresasAManifestar = $this->dfe51000ImportacaoConsultaNotaFiscalService->getManifestacaoService()->getManifestacaoFactory()->getEmpresasAgendamento();
        
        //Inicio manifestacao, importacao NFe e envio de e-mail dos itens pendente de relacionamento produto.
        $this->manifestaImportaNFeAgendamento($empresasAManifestar, $nrorg);
        
        //Configura a manifestacao CTe.
        $this->sincronizacaoCTEService->setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $this->fileUploader);
        
        //Inicio manifestacao e importacao CTe.
        $this->manifestaImportaCTeAgendamento($empresasAManifestar, $nrorg);
        
        var_dump('FIM agendamento.');
    }
    
    private function setAgendamentoManifestacao($nrorg, $useVpd, &$newEntityManager) {
        if($useVpd) {
            $newEntityManager = $this->connectionCustom->updateVpdConnection($nrorg);
            
            if(empty($newEntityManager)) throw new \Exception('Falha ao gerar o newEntityManager.');
        }
        if($nrorg > 1) $this->setFileServerAgendamento($nrorg);
    }
    
    private function setFileServerAgendamento($nrorg) {
        $httpInterface = new cURLRequest(self::PRODUCTION_SERVER_URL);
        $objFileUploaderAPIKey = InstanceManager::getInstance()->getService('fileUploaderAPIKeyProvider');
        $apiKey = (array) $objFileUploaderAPIKey;
        $hashApiKey = '';
        
        foreach($apiKey as $key => $conteudo) {
            if(strpos($key, 'apiKeyMapping') !== false) {
                if(array_key_exists($nrorg, $conteudo)) {
                    $hashApiKey = $conteudo[$nrorg];
                    $this->fileUploader = new fileUploader($httpInterface, $hashApiKey , []);
                    break;
                }
                else {
                    throw new \Exception('Fileserver nao esta configurado corretamente para o NRORG: '.$nrorg);
                    break;
                }
            }
            else break;
        }
    }
    
    private function manifestaImportaNFeAgendamento($empresasAManifestar, $nrorg) {
        var_dump('A manifestacao NFe sera iniciada.');
        
        foreach($empresasAManifestar as $empresa) {
            try{
                var_dump('Inicio sincronizacao NFe. Empresa: '.$empresa['CDEMPRESA'].' e CNPJ: '.$empresa["NRINSJURIEST"]);
                
                $config = $this->dfe51000ImportacaoConsultaNotaFiscalService->getManifestacaoService()->montaConfigManifestacao
                          ($empresa['CDEMPRESA'], $nrorg, null, $empresa["NRINSJURIEST"]);
                
                if(empty($config)) throw new \Exception('Erro na geracao do config.');
                
                $params = ["FORMAPESQUISA" => '0', "CDINSCESTA" => $config->ie];
                
                $sincronizacao = $this->dfe51000ImportacaoConsultaNotaFiscalService->getManifestacaoService()->sincronizarNfeManifestacao($config, $params);
                var_dump('Resultado da sincronizacao NFe.', $sincronizacao);
                
            } catch (\Exception $e) {
                if($e->getMessage() == "Nenhum documento localizado.") var_dump($e->getMessage());
                else var_dump('Ocorreu um erro na sincronizacao NFe.', $e->getMessage());
            }
        }
        var_dump('FIM Manifestacao NFe');
        
        try {
            var_dump('A importacao NFe sera iniciada.');
            $resp = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarNotas();
            var_dump('Resultado da importacao NFe:', $resp);
            
            $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
            var_dump('Erros da importacao NFe:', $erros);
            
            if($resp['SUCESSOS'] > 0) {
                var_dump('Inicio envio e-mail dos itens pendente de relacionamento produto.');
                
                $emailPendente = $this->dfe51000ImportacaoConsultaNotaFiscalService->enviaEmailRelacionamentoPendente();
                var_dump('Resultado envio e-mail:', $emailPendente);
            }
        } catch (\Exception $e) {
            if($e->getMessage() == "Não há notas no diretório.") var_dump($e->getMessage());
            else var_dump('Ocorreu erro na importacao NFe.', $e->getMessage());
        }
        var_dump('FIM Importacao NFe');
    }
    
    private function manifestaImportaCTeAgendamento($empresasAManifestar, $nrorg) {
        var_dump('A manifestacao CTe sera iniciada.');
        
        foreach($empresasAManifestar as $empresa) {
            try{
                var_dump('Inicio sincronizacao CTe. Empresa: '.$empresa['CDEMPRESA'].' e CNPJ: '.$empresa["NRINSJURIEST"]);
                
                $config = $this->sincronizacaoCTEService->montaConfigCte($empresa['CDEMPRESA'], $nrorg, null, $empresa["NRINSJURIEST"]);
                
                if(empty($config)) throw new \Exception('Erro na geracao do config.');
                
                $sincronizacao = $this->sincronizacaoCTEService->sincronizarCte($config);
                var_dump('Resultado da sincronizacao CTe.', $sincronizacao);
                
            } catch (\Exception $e) {
                if($e->getMessage() == "Nenhum documento localizado.") var_dump($e->getMessage());
                else var_dump('Ocorreu um erro na sincronizacao CTe.', $e->getMessage());
            }
        }
        var_dump('FIM Manifestacao CTe');
        
        try {
            var_dump('A importacao CTe sera iniciada.');
            $resp = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarCTe();
            var_dump('Resultado da importacao CTe:', $resp);
            
            $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
            var_dump('Erros da importacao CTe:', $erros);
            
        } catch (\Exception $e) {
            if($e->getMessage() == "Não há notas no diretório.") var_dump($e->getMessage());
            else var_dump('Ocorreu erro na importacao CTe.', $e->getMessage());
        }
        var_dump('FIM Importacao CTe');
    }
    
    public function dadosAdic_XML(DTO\Request\Row $request, DTO\Response $response){
       $row = $request->getRow();
       $nracessonfe = $row["NRACESSONFE"];
       $result = $this->dfe51000ImportacaoConsultaNotaFiscalService->dadosAdic_XML($nracessonfe);
       $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('dadosAdic_XML', array($result)));
    }
}