<?php

namespace Teknisa\DF\Services;

use Zeedhi\Framework\DTO;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\Utilities;
use Teknisa\DF\Factories\AprovacaoLancamentosFactory;
use Teknisa\DF\Services\ContabilizacaoService;
use Teknisa\DF\Services\ValidacoesEstoqueService;
use Teknisa\DF\Factories\ProceduresNfFactory;
use Teknisa\Model\Fiscal\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Contabilidade\Entities\Contopectbl;
use Teknisa\DF\Modules\Entrada\Services\ManifestacaoDestinatarioService;
use Fiscal\Entities\Rateionf;
use Fiscal\Entities\Notafiscal;
use Geral\Entities\Itemproduto;
use Geral\Entities\Produto;
use Geral\Entities\Prodfili;
use Zeedhi\Framework\ORM;

use Teknisa\DF\Factories\NotafiscalLancamentoFactory;

class AprovacaoLancamentosService
{
    /*---------    CÓDIGOS DE ERRO CANCELAMENTO ----------*/
    private static $array_erros_cancelamento = array(
        '1' => 'Documento possui boletos baixados',
        '2' => 'Documento possui vencimentos parcelados',
        '3' => 'Documento com vencimentos prorrogados',
        '4' => 'Documento com ajustes em boletos',
        '5' => 'Documento com boleto real',
        '6' => 'Documento relacionado à um boleto CNAB',
        '7' => 'Documento com boleto associado',
        '8' => 'Documento possui crédito já utilizado em um boleto financeiro',
        '9' => 'Data do lançamento fora do período contábil',
        '10' => 'Documento possui devolução de transferência do OL',
        '11' => 'Mapa de apuração fiscal bloqueado',
        '12' => 'NFE transmitida, aceita ou uso denegado',
        '13' => 'Documento com impostos apurados no mapa de lucro presumido',
    );
    
    const MSG_ERROS_AUDITORIA = array(
        "BLOQ"          => "BLOQ",
        "BLOQ_BC"       => "BLOQ_BC",
        "DTLANC_E"      => "DTLANC_E",
        "DTLANC_S"      => "DTLANC_S",
        "LANCAPUR"      => "LANCAPUR",
        "LANCAPURSN"    => "LANCAPURSN",
        "MPLUCPRE"      => "MPLUCPRE",
        "IMPXML"        => "IMPXML",
        "IMPOBR"        => "IMPOBR",
        "CST_IMP"       => "CST_IMP" ,
        "ERRO_CST"      => "ERRO_CST",
        "CFOP_IMP"      => "CFOP_IMP",
        "ERRO_CFOP"     => "ERRO_CFOP",
        "ALTERA_FIN"    => "ALTERA_FIN",
        "BLOCK_NOTE"    => "BLOCK_NOTE" 
    );
    
    protected $errosContabilizacao;

    public function __construct(NewCode $novoCodigo,
                                EntityManager $entityManager,
                                Utilities $util,
                                Environment $environment,
                                AprovacaoLancamentosFactory $aprovacaoLancamentosFactory,
                                ProceduresNfFactory $proceduresNfFactory,
                                ValidacoesEstoqueService $validacoesEstoqueService,
                                ContabilizacaoService $contabilizacaoService,
                                ManifestacaoDestinatarioService $manifestacaoService)
    {
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->util = $util;
        $this->environment = $environment;
        $this->aprovacaoLancamentosFactory = $aprovacaoLancamentosFactory;
        $this->proceduresNfFactory = $proceduresNfFactory;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->contabilizacaoService = $contabilizacaoService;
        $this->manifestacaoService  = $manifestacaoService;
        $this->lancamentoFactory = new NotafiscalLancamentoFactory($entityManager);
    }

    public function aprovar($organizacao, $filial, $lancamento, $entrada_saida, $response){
        try {
            $notafiscal = $this->entityManager->getRepository('Fiscal\Entities\Notafiscal')->findOneBy(array('nrorg' => $organizacao, 'identrsaidop' => $entrada_saida, 'cdfilial' => $filial, 'nrlanctonf' => $lancamento));
            $tipooperac = $this->entityManager->getRepository('Geral\Entities\Tipooperac')->findOneBy(array('nrorg' => $organizacao, 'identrsaidop' => $entrada_saida, 'cdtipooper' => $notafiscal->getCdtipooper()));
            
            $errosAprovacao = $this->validacaoGeral($notafiscal, $tipooperac);

            if(!empty($errosAprovacao))
                throw new \Exception('ERRO_VALIDACAO');

            $errosAprovacao = $this->aprovarNota($notafiscal, $tipooperac, $response);

            if(!empty($errosAprovacao))
                throw new \Exception('ERRO_APROVACAO');

            return array();
        } catch (\Exception $e) {
            if(!empty($errosAprovacao))
                return $errosAprovacao;

            throw $e;
        }
    }
    
    public function auditar($organizacao, $filial, $lancamento, $entrada_saida, $response){ 
        try {
            $notafiscal = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notafiscal')->findOneBy(array('nrorg' => $organizacao, 'identrsaidop' => $entrada_saida, 'cdfilial' => $filial, 'nrlanctonf' => $lancamento));
            $tipooperac = $this->entityManager->getRepository('Geral\Entities\Tipooperac')->findOneBy(array('nrorg' => $organizacao, 'identrsaidop' => $entrada_saida, 'cdtipooper' => $notafiscal->getCdtipooper()));

            $errosAuditoria = $this->auditarNota($notafiscal, $tipooperac, true, $response);
            
            if(!empty($errosAuditoria))
                throw new \Exception('ERRO_AUDITAR');

            return array();
        }
        catch (\Exception $e) {
            if(!empty($errosAuditoria))
                return $errosAuditoria;

            throw $e;
        }
    }

    public function cancelar($organizacao, $filial, $lancamento, $entrada_saida){
        $notafiscal = $this->entityManager->getRepository('Fiscal\Entities\Notafiscal')->findOneBy(array('nrorg' => $organizacao, 'identrsaidop' => $entrada_saida, 'cdfilial' => $filial, 'nrlanctonf' => $lancamento));
        $this->validaDevolucaoTransferencia($notafiscal);
        if($this->validaMapa($notafiscal)) {
            return "Operação Bloqueada! Existem mapas bloqueados relacionados a nota fiscal.";   
        }
        return $this->cancelarAprovacao($notafiscal);
    }

    private function validaDevolucaoTransferencia($notafiscal){
        $itensProd = $this->entityManager->getRepository('Geral\Entities\Itemproduto')->findBy(array('nrorg' => $notafiscal->getNrorg(), 'cdfilial' => $notafiscal->getCdfilial(), 'nrlanctonf' => $notafiscal->getNrlanctonf()));

        foreach ($itensProd as $item) {
            $iddevolucaool = $item->getIddevolucaool();
            if ($iddevolucaool === 'S') {
                throw new \Exception('Cancelamento bloqueado! Documento possui devolução de transferência do OL.');
            }
        }
    }
    
        
    private function validaMapa($notafiscal) {
        $mapas = $this->aprovacaoLancamentosFactory->getMapasBloqueados($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        
        return (count($mapas) > 0);
        
    }

    private function cancelarAprovacao($notafiscal){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $erros = $this->proceduresNfFactory->procedureCancelaDadosFinanceiro($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());

            if ($erros['P_RETORNO'] != 0) {
                throw new \Exception();
            }

            self::deletaContabilizacao($notafiscal->getNrorg(), $notafiscal->getCdempresa(), $notafiscal->getNrlanctbdoc());
            self::deletaMapa($notafiscal);
            self::atualizaStatusNotaFiscalCancelada($notafiscal);
            $connection->commit();
            return $erros['P_RETORNO'];
        } catch (\Exception $e) {
            $connection->rollback();
            return self::$array_erros_cancelamento[$erros['P_RETORNO']];
        }
    }

    /**
     * Deleta a contabilização da nota fiscal
     */
    private function deletaContabilizacao($organizacao, $empresa, $nrlanctbdoc)
    {
        $lancamentoContabil = $this->entityManager->getRepository('Geral\Entities\Lanccont')->findBy(array('nrorg' => $organizacao, 'cdempresa' => $empresa, 'nrlanccont' => $nrlanctbdoc));

        if ($lancamentoContabil) {
            foreach ($lancamentoContabil as $lancamento)
                $lancamento->delete();
        }

        $this->entityManager->flush();
    }
    
    /**
     * Deleta tabela ITMAPARECUP relacionada
     */
     private function deletaMapa($notafiscal){
        $mapa = $this->aprovacaoLancamentosFactory->delMapa($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        
    }

    /**
     * Atualiza os status da nota fiscal para CANCELADA
     */
    private function atualizaStatusNotaFiscalCancelada($notafiscal)
    {
        $notafiscal->setIdstatus('P');
        $notafiscal->setIdbloqueado('N');
        $notafiscal->setCdopecancnf($this->environment->getCdOperador());
        $notafiscal->setDtaprovacao(null);
        $notafiscal->setDtexportacao(null);
        $notafiscal->setIddocfinanc('N');
        $notafiscal->setIdexpnfep('N');
        $notafiscal->setIdexpnfemanaus('N');
        $notafiscal->setIdexpnfesanand('N');
        $notafiscal->setIdverifnfconf('N');
        $notafiscal->setNrseqcapadoc('');
        $notafiscal->setNrlanctbdoc(null);
        $notafiscal->setNrlanctbajus(null);

        $notafiscal->setDtultatu(DateUtil::getDataAtual());
        $notafiscal->setCdoperultatu($this->environment->getCdOperador());

        $this->entityManager->flush();
    }

    public function validacaoGeral($notafiscal, $tipooperac) {
        try {
            self::validaModDocFisc($notafiscal->getCdmoddocfisc());
            self::validaNotaFiscEstornada($notafiscal->getDsnfestcanc());
            self::validaIdErroNotaFisc($notafiscal->getIderros());
            self::validaTipoDeConta($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg(), $notafiscal->getIdentrsaidop(), $tipooperac);
            self::validaDataIntegracao($notafiscal->getCdempresa(), $notafiscal->getIdentrsaidop(), $tipooperac, $notafiscal->getDtentrsaid(), $notafiscal->getNrorg());
            return self::validaConsisteFiscal($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrnotafisc(), $notafiscal->getVrnotafisc(), $notafiscal->getDtemissao());
        } catch (\Exception $e ) {
            return array(array('NRNOTAFISC' => $notafiscal->getNrnotafisc() ? $notafiscal->getNrnotafisc() : null, 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => ($notafiscal->getDtemissao()) ? $notafiscal->getDtemissao()->format('d/m/Y') : '', 'DSERRO' => $e->getMessage()));
        }
    }
    
    public function getExisteBoleto($cdfilial, $nrlanbolpag){
        return $this->lancamentoFactory->existeBoleto($cdfilial, $nrlanbolpag);
    }

    private function aprovarNota($notafiscal, $tipooperac, $response){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $erroDadosFinanceiros = '';
            if ($tipooperac->getIdutilvenc() === 'S') {
                $cdfilial = $notafiscal->getCdfilial();
                $nrlanctonf = $notafiscal->getNrlanctonf();
                $nrorg = $notafiscal->getNrorg();
                $identrsaidop = $notafiscal->getIdentrsaidop();
                $cdtipooper = $notafiscal->getCdtipooper();
                $prodNota = $this->aprovacaoLancamentosFactory->getProdutosNota($cdfilial, $nrlanctonf);
                $tpOperacTpCta = $this->aprovacaoLancamentosFactory->getTipooperacCta($identrsaidop, $cdtipooper, $nrorg);
                if($identrsaidop == 'E') {
                    $temLancTpCtaPg = $this->aprovacaoLancamentosFactory->getVenctosNotaEnt($cdfilial, $nrlanctonf, $nrorg);
                    if (!empty($temLancTpCtaPg)) {
                        $tpCtPagarProd = $this->aprovacaoLancamentosFactory->getTipoContaPagarProd($cdfilial, $nrlanctonf, $nrorg);
                        $temAFtpCtaPg = $this->aprovacaoLancamentosFactory->getAutoForne($cdfilial, $nrlanctonf, $nrorg);
                        $temVectoTpCtPg = $this->aprovacaoLancamentosFactory->verificaTipoContaPagar($cdfilial, $nrlanctonf, $nrorg);
                        $temFornTpCtPg = $this->aprovacaoLancamentosFactory->getTipoContaPagarForn($cdfilial, $nrlanctonf, $nrorg);
                        $qtdeLancTpCtaPg = (count($temLancTpCtaPg));
                        $prodNotaSemTpCtPagar = false;
                        $notaComVectoSemTpCtPagar = false;
                        
                        foreach($prodNota as $produto) {
                            if(array_search($produto["CDPRODUTO"], array_column($tpCtPagarProd, "CDPRODUTO")) === false ) {
                                $prodNotaSemTpCtPagar = true;
                                break;
                            }
                        }
                        foreach($temLancTpCtaPg as $lancTpCtaPg) {
                            if($lancTpCtaPg["CDTPCTPAGAR"] == null) {
                                $notaComVectoSemTpCtPagar = true;
                                break;
                            }
                        }
                        if($prodNotaSemTpCtPagar && $tpOperacTpCta["CDTPCTPAGAR"] == null && $temFornTpCtPg["CDTPCTPAGAR"] == null 
                           && $temAFtpCtaPg["CDTPCTPAGAR"] == null && $temVectoTpCtPg["CDTPCTPAGAR"] == null ) {
                                throw new \Exception("TIPO_CONTA_PAGAR_N_PARAM_GER");
                                                      
                            }
                        else if($notaComVectoSemTpCtPagar && $qtdeLancTpCtaPg > 1) {
                            if($prodNotaSemTpCtPagar && $tpOperacTpCta["CDTPCTPAGAR"] == null && $temFornTpCtPg["CDTPCTPAGAR"] == null
                               && $temAFtpCtaPg["CDTPCTPAGAR"] == null ) {
                                throw new \Exception("TIPO_CONTA_PAGAR_N_PARAM_GER");
                            }
                        }
                    }
                    else {
                        throw new \Exception('NOTAFISCAL_SEMVENCIMENTO_COMTIPOOP');
                    }
                } else { //Validações de NF de Saída.
                    $temLancTpCtaReceb = $this->aprovacaoLancamentosFactory->getVenctosNotaSaida($cdfilial, $nrlanctonf, $nrorg);
                    
                    if(!empty($temLancTpCtaReceb)) {
                        $tpCtReceberProd = $this->aprovacaoLancamentosFactory->getTipoContaReceberProd($cdfilial, $nrlanctonf, $nrorg);
                        $prodNotaSemTpCtReceber = false;
                    
                        foreach($prodNota as $produto) {
                            if(array_search($produto["CDPRODUTO"], array_column($tpCtReceberProd, "CDPRODUTO")) === false ) {
                                $prodNotaSemTpCtReceber = true;
                                break;
                            }
                        }
                        if($prodNotaSemTpCtReceber && $tpOperacTpCta["CDTPCTRECEB"] == null) {
                            throw new \Exception('TIPO_CONTA_PAGAR_N_PARAM_PROD_TIPOP');
                        }
                    }
                    else {
                        throw new \Exception('NOTAFISCAL_SEMVENCIMENTO_COMTIPOOP');
                    }
                }
                
                $filiemit = $notafiscal->getIdentrsaidop() === 'E' ? $notafiscal->getCdfilirece() : $notafiscal->getCdfiliemit();
                try {
                    $erroDadosFinanceiros = $this->proceduresNfFactory->procedureGeraDadosFinanceiro($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), 'A', $filiemit);
                } catch (\Exception $e) {
                    $msgError = $e->getMessage();
                    $cdCLASSFINAError = strpos($msgError, 'CDCLASSFINA');
                    
                    if($cdCLASSFINAError !== false) 
                        throw new \Exception("TIPO_CONTA_PAGAR_N_PARAM_GER");
                    else 
                        throw new \Exception('ERRO_FINANCEIRO');
                }
            }
            if ($erroDadosFinanceiros != '' && $erroDadosFinanceiros['P_RESULTADO']) {
                switch ($erroDadosFinanceiros['P_RESULTADO']) {
                    case 'NF_NOT_EXIST' :
                        throw new \Exception('NF_NOT_EXIST');
                        break;
                    case 'CNT_SUP_VRORC' :
                        throw new \Exception("CNT_SUP_VRORC");
                        break;
                    case 'INV_TP_OP' :
                        throw new \Exception("INV_TP_OP");
                        break;
                    case substr($erroDadosFinanceiros['P_RESULTADO'], 0, 16) == 'NOT_CENTCUSTFIL_' :
                        $str = str_split(substr($erroDadosFinanceiros['P_RESULTADO'], 16, 8), 4);
                        throw new \Exception("CENTRO_CUSTO_NAO_INFORMADO_FILIAL");
                        break;
                }
            }
                        
            try {
                $this->contabilizacaoService->contabilizaNotasFiscais($notafiscal->getCdempresa(), $notafiscal->getIdentrsaidop(), $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
            }
            catch(\Exception $e) {
                $this->errosContabilizacao = $this->contabilizacaoService->getErrosContabilizacao();
                throw new \Exception('ERRO_CONTABILIZACAO');
            }

            self::atualizaStatusNotaFiscalAprovada($notafiscal, $tipooperac);
            
            $connection->commit();
            
            $this->manifestaNaAprovacao($notafiscal->getNracessonfe(), $notafiscal->getCdempresa(), $notafiscal->getCdinscesta());
        
            
        } catch (\Exception $e) {
            
            $connection->rollback();
            
            $this->contabilizacaoService->saveContabilizacaoData($response);
            
            if ($erroDadosFinanceiros != '' && $erroDadosFinanceiros['P_RESULTADO']) {
                return array(array('NRNOTAFISC' => $notafiscal->getNrnotafisc(), 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => $notafiscal->getDtemissao()->format('d/m/Y'), 'DSERRO' => $e->getMessage(), 'TIPODEERRO' => '1'));
            }
            else if (!empty($this->errosContabilizacao)) {
                $erros = array();
                foreach($this->errosContabilizacao as $erroContabilizacao) {
                    $erros[] = array('NRNOTAFISC' => $notafiscal->getNrnotafisc(), 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => $notafiscal->getDtemissao()->format('d/m/Y'), 'DSERRO' => $erroContabilizacao['DSERROTEMP'], 'TIPODEERRO' => '4');
                }
                return $erros;
            }
            else {
                 return array(array('NRNOTAFISC' => $notafiscal->getNrnotafisc(), 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => $notafiscal->getDtemissao()->format('d/m/Y'), 'DSERRO' => $e->getMessage(), 'TIPODEERRO' => '1'));
            }
            
        }
    }
    
    private function auditarNota($notafiscal, $tipooperac, $atualizaCFOP, $response) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $statusNf = $this->lancamentoFactory->getStatusNota($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getNrorg());
            
            if(!in_array($statusNf['IDSTATUSNFE'], ['P', 'R']) || $statusNf['IDBLOQUEADO'] === 'S') { //Só permite alterar nota pendente ou rejeitada e não bloqueada.
                throw new \Exception(self::MSG_ERROS_AUDITORIA["BLOCK_NOTE"]);
            }
            $this->validacoesAuditoria($notafiscal);

            if($notafiscal->getIdentrsaidop() == 'E' 
                || ($notafiscal->getIdentrsaidop() == 'S' 
                    && ($notafiscal->getIdstatusnfe() == 'P' || $notafiscal->getIdstatusnfe() == 'R') )) {
                    
                $this->fechaNotaFiscal($notafiscal, $tipooperac);
            }
                    
            if($notafiscal->getIdentrsaidop() == 'S' || ($notafiscal->getIdstatus() == 'A' || $notafiscal->getIdStatus() == 'E')) {
     
                if($tipooperac->getIdcontaboper() == 'S') {
                    
                    try {
                        $this->contabilizacaoService->contabilizaNotasFiscais($notafiscal->getCdempresa(), $notafiscal->getIdentrsaidop(), $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
                    }
                    catch(\Exception $e) {
                        $this->errosContabilizacao = $this->contabilizacaoService->getErrosContabilizacao();
                        throw new \Exception('ERRO_CONTABILIZACAO');
                    }
                }   
            }
            
            $connection->commit();
        }
        catch(\Exception $e) {
            $connection->rollback();
            $dtemissao = '';
            $dtemissao = $notafiscal->getDtemissao();
            
            if($dtemissao) {
                $dtemissao = $dtemissao->format('d/m/Y');
            }
            
            if($e->getMessage() == self::MSG_ERROS_AUDITORIA["BLOQ_BC"]) {
                return array(array('CDFILIAL' => $notafiscal->getCdfilial(), "NRLANCTONF" => $notafiscal->getNrlanctonf(), 'NRNOTAFISC' => $notafiscal->getNrnotafisc(), 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => $dtemissao, 'DSERRO' => $e->getMessage(), 'TIPODEERRO' => '6'));
            }
            else if (!empty($this->errosContabilizacao)) {
                $erros = array();
                foreach($this->errosContabilizacao as $erroContabilizacao) {
                    $erros[] = array('CDFILIAL' => $notafiscal->getCdfilial(), "NRLANCTONF" => $notafiscal->getNrlanctonf(), 'NRNOTAFISC' => $notafiscal->getNrnotafisc(), 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => $dtemissao, 'DSERRO' => $erroContabilizacao['DSERROTEMP'], 'TIPODEERRO' => '4');
                }
                return $erros;
            }
            else {
                return array(array('CDFILIAL' => $notafiscal->getCdfilial(), "NRLANCTONF" => $notafiscal->getNrlanctonf(), 'NRNOTAFISC' => $notafiscal->getNrnotafisc(), 'VRNOTAFISC' => $notafiscal->getVrnotafisc(), 'DTEMISSAO' => $dtemissao, 'DSERRO' => $e->getMessage(), 'TIPODEERRO' => '5'));
            }
        }
    }
    
    /**
     * Validações Auditoria 
     */ 
    public  function validacoesAuditoria($notafiscal) {
        $this->validaBloqueioNotaOperacao($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        $this->validaBloqueioBC($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        $this->validaBloqueioMapa($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        $this->validaBloqueioMapaSimples($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        $this->validaDataLancamento($notafiscal->getCdempresa(), $notafiscal->getIdentrsaidop(), $notafiscal->getDtemissao(), $notafiscal->getDtentrsaid(), $notafiscal->getIdstatus());
        $this->validaExisteMapaLucroPresumido($notafiscal->getNrsequmapapunf());
        $this->validaNotaImportadaViaXML($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getIdnfgeradaviaxml(), $notafiscal->getNrorg());
        
    }
    
    private function validaBloqueioNotaOperacao($cdfilial, $nrlanctonf) {
        try {
            $this->aprovacaoLancamentosFactory->bloqueiaOperadorNotaFiscal($cdfilial, $nrlanctonf);
        }
        catch(\Exception $e) {
            throw new \Exception(self::MSG_ERROS_AUDITORIA["BLOQ"]);
        }
    }
    
    private function validaBloqueioBC($cdfilial, $nrlanctonf) {
        if($this->aprovacaoLancamentosFactory->existeNotaBCMensal($cdfilial, $nrlanctonf)) {
            throw new \Exception(self::MSG_ERROS_AUDITORIA["BLOQ_BC"]);
        }
    }
    
    private function validaBloqueioMapa($cdfilial, $nrlanctonf) {
        if($this->aprovacaoLancamentosFactory->existeMapaApuradoBloqueado($cdfilial, $nrlanctonf)) {
            throw new \Exception(self::MSG_ERROS_AUDITORIA["LANCAPUR"]);
        }
    }
    
    private function validaBloqueioMapaSimples($cdfilial, $nrlanctonf) {
        if($this->aprovacaoLancamentosFactory->existeMapaSimplesBloqueado($cdfilial, $nrlanctonf)) {
            throw new \Exception(self::MSG_ERROS_AUDITORIA["LANCAPURSN"]);
        }
    }
    
    private function validaDataLancamento($cdempresa, $identrsaiop, $dtemissao, $dtentrsaid, $idstatus) {
        $data = $identrsaiop == 'S' ? $dtemissao : $dtentrsaid;
    
        if($idstatus == 'A' || $idstatus == 'E') {
            if(!$data || !$this->aprovacaoLancamentosFactory->validaDataLancamento($cdempresa, $data->format("d/m/Y"))) {
                $mensagem = $identrsaiop == 'S' ? self::MSG_ERROS_AUDITORIA["DTLANC_S"]
                                                : self::MSG_ERROS_AUDITORIA["DTLANC_E"];
                throw new \Exception($mensagem);
            }
        }
    }
    
    private function validaExisteMapaLucroPresumido($nrsequmapapunf) {
        if($nrsequmapapunf) {
            if($this->aprovacaoLancamentosFactory->existeMapaLucroPresumido($nrsequmapapunf)) {
                throw new \ExcePtion(self::MSG_ERROS_AUDITORIA["MPLUCPRE"]);
            }
        }
    }
    
    private function validaNotaImportadaViaXML($cdfilial, $nrlanctonf, $idnfgeradaviaxml, $nrorg) {

        if($idnfgeradaviaxml == 'S') {
            $paramgeral = $this->entityManager->getRepository('Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $nrorg));

            if($paramgeral->getIdgernfigualxml() == 'S') throw new \Exception(self::MSG_ERROS_AUDITORIA["IMPXML"]);
        }
        
    }
    
    public function validaNumero($row){
        $nrnotafisc = $row['NRNOTAFISC'];
        $lancamento = $row['CDFILIAL'].$row['NRLANCTONF'];
        $cdserie    = array_key_exists('CDSERIE', $row) ? $row['CDSERIE'] : $row['CDSERIEENTR'];
        $cdmoddocfisc    = $row['CDMODDOCFISC'];
        $identrsaidop = $row['IDENTRSAIDOP'];
        $nrinsjurfili = $row['NRINSJURFILI'];
        
        if(sizeof($this->aprovacaoLancamentosFactory->getNumeros($nrnotafisc, $lancamento, $cdserie, $cdmoddocfisc, $identrsaidop, $nrinsjurfili)) > 0){
            throw new \Exception("Número de nota fiscal já informado em outro documento");
        } else{
            return 'S';
        }
    }
    
    public function validaDataEmissao($row){
        $dtemissao  = ORM\DateTime::createFromFormat('d/m/Y', substr($row['DTEMISSAO'], 0, 10));
        $dtentrsaid = ORM\DateTime::createFromFormat('d/m/Y', substr($row['DTENTRSAID'], 0, 10));
        
        if($dtemissao > $dtentrsaid){
            throw new \Exception("Data de emissão deve ser menor ou igual a data de saida");
        } else{
            $cdfilial = $row['CDFILIAL'];
            $nrlanctonf = $row['NRLANCTONF'];
            $cdempresa = $row['CDEMPRESA'];
            
            $imposBloqDtemissao = $this->aprovacaoLancamentosFactory->getImposBloqDtemissao($cdfilial, $nrlanctonf, $cdempresa, $row["DTEMISSAO"]);
            
            if(sizeof($imposBloqDtemissao) > 0){
                $message = join(', ',$imposBloqDtemissao);
                throw new \Exception("Data de emissão informada pertence a um mapa de apuração fiscal bloqueado para o(s) imposto(s)". $mimpostos);
            } else{
                return 'S';
            }
        }
    }
    
    // Fim das Validações da Auditoria
    
    private function fechaNotaFiscal($notafiscal, $tipooperac) {

        $rateioAnterior = array();
        
        $filial = $notafiscal->getIdentrsaidop() == 'E' ? $notafiscal->getCdfilirece() : $notafiscal->getCdfiliemit();
        

        
        $fornCli = $notafiscal->getIdorigdestnf() == 'F' ? $notafiscal->getCdforneced() 
                                                             : ($notafiscal->getIdorigdestnf() == 'C'? $notafiscal->getCdcliente() 
                                                             : $notafiscal->getCdfiliemit());
        
        $paramgeral = $this->entityManager->getRepository('Geral\Entities\Paramgeral')->findOneBy(array('nrorg' => $notafiscal->getNrorg()));

        $this->aprovacaoLancamentosFactory->deleteTrabimposNfTot($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());

        $lanctoestoq = $this->entityManager
                     ->getRepository('Estoque\Entities\Lanctoestoq')
                     ->findOneBy(array('nrorg' => $notafiscal->getNrorg(), 'idtplancto' => $notafiscal->getIdentrsaidop(), 
                                       'cdfilial' => $notafiscal->getCdfilial(), 'nrlanctonf' => $notafiscal->getNrlanctonf()));
                                       
        $nrlancestq = $lanctoestoq ? $lanctoestoq->getNrlancestq() : '';
        
        $valorRetidoAntes   = $this->aprovacaoLancamentosFactory->getValorTotalImpostoRetido($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
            
        $valorNotaAntes     = $notafiscal->getVrnotafisc();

        $produtos   = $this->aprovacaoLancamentosFactory->getProdutosNota($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());

        $impostosObrigatorio = $this->aprovacaoLancamentosFactory->getImpostoObrigatorio($notafiscal->getCdtipooper(), $notafiscal->getIdentrsaidop(), $filial);
    
        if($notafiscal->getIdentrsaidop() == 'S') {
            $valorNotaDepois = $this->proceduresNfFactory->procedureImpostoSaida($notafiscal->getCdEmpresa(), $notafiscal->getCdinscesta(), $notafiscal->getCdtipooper(),
                                                       $fornCli, $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), 
                                                       $notafiscal->getDtentrsaid()->format('d/m/Y'), 
                                                       $this->environment->getCdOperador(), $nrlancestq);
        
        }
    
        
        foreach($produtos as $produto) {
           
            $impostosNota = array();
                                                             
            $cdcliente    = $notafiscal->getIdorigdestnf() == 'C' ? $fornCli : '';
            $cdfornecedor = $notafiscal->getIdorigdestnf() == 'F' ? $fornCli : '';
            
            if($notafiscal->getIdentrsaidop() == 'E')  {
                $valorProduto  = $produto["VRTOTAITEM"] - $produto["VRDESCITEM"];
                
                $impoInfoMan = $this->aprovacaoLancamentosFactory->getImpoInfoMan($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto["NRSEQITPROD"]);
            
                $grupoFiscal   = $this->proceduresNfFactory->procedureConsultaGrupoFiscal($notafiscal->getIdorigdestnf(), $fornCli, $notafiscal->getCdfilial(), $notafiscal->getIdentrsaidop(), $notafiscal->getCdtipooper());
                $interEstadual = $this->aprovacaoLancamentosFactory->operacaoInterEstadual($notafiscal->getCdfilial(), $notafiscal->getCdempresa(), $notafiscal->getCdinscesta(), $cdcliente, $cdfornecedor, $notafiscal->getSgestado());

                $this->proceduresNfFactory  
                 ->procedureImpostoEntradaProduto($notafiscal->getCdempresa(), $notafiscal->getCdinscesta(), $notafiscal->getCdtipooper(), $notafiscal->getCdforneced(),
                                           $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getDtentrsaid()->format('d/m/Y'), $notafiscal->getIdentrsaidop(),
                                           $this->environment->getCdOperador(), $grupoFiscal, $interEstadual, $produto["CDARVPROD"], '',  $notafiscal->getIdorigdestnf(),
                                           $notafiscal->getCdfilirece(), $notafiscal->getCdfiliemit(), $notafiscal->getCdcliente(), $valorProduto, $produto["IDORIGEMPROD"],
                                           $notafiscal->getSgestado(), $notafiscal->getCdmunicipio());
                                           
                $aliqUniFis  = $this->aprovacaoLancamentosFactory->getAliqUniFis($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
                
                $impostosManuaisInseridos = array();
                $impostosInseridos = array();
            
                foreach($impoInfoMan as $impostoManual) {
                    if($impostoManual["IDPRODNOTA"] == "P") {
                        $this->aprovacaoLancamentosFactory
                             ->deleteTrabitproimpos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto["NRSEQITPROD"], $impostoManual["CDIMPOSTO"], "1");
                        
                        $this->aprovacaoLancamentosFactory
                             ->insertTrabitproimpos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto["NRSEQITPROD"], $impostoManual["CDIMPOSTO"], 
                                                    '1', $impostoManual["VRALIQITEM"], $impostoManual["VRPERREDBCIT"], $impostoManual["VRBASECALCIT"], $impostoManual["VRIMPOSTOIT"],
                                                    $impostoManual["IDVALORINFIT"], $impostoManual["VRADICALIQFECP"], $impostoManual["VRADICFECP"]);
                        
                    }
                    else {
                        $this->aprovacaoLancamentosFactory->deleteTrabimposNfTotImposto($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoManual["CDIMPOSTO"]);
                        $this->aprovacaoLancamentosFactory->insereTrabimposNfTot($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoManual["CDIMPOSTO"], 0);
                    }
                    $this->aprovacaoLancamentosFactory->deleteAliqTemp($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoManual["CDIMPOSTO"]);
                    $this->aprovacaoLancamentosFactory->deleteInciTemp($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoManual["CDIMPOSTO"]);
                    
                    $impostosManuaisInseridos[$impostoManual["CDIMPOSTO"]] = $impostoManual["CDIMPOSTO"];
    
                    $impostosInseridos[] = array("CDIMPOSTO" => $impostoManual["CDIMPOSTO"]);

                }
                 
                foreach($aliqUniFis as $impostoRegra) {
                    
                    if(!isset($impostosManuaisInseridos[$impostoRegra["CDIMPOSTO"]])) {
                        
                        if($impostoRegra["IDPRODNOTA"] == "P") {
                            $this->aprovacaoLancamentosFactory
                                 ->deleteTrabitproimpos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto["NRSEQITPROD"], $impostoRegra["CDIMPOSTO"], "1");
                        
                            $this->aprovacaoLancamentosFactory
                                 ->insertTrabitproimpos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto["NRSEQITPROD"], $impostoRegra["CDIMPOSTO"], 
                                                        '1', $impostoRegra["VRALIQUOTA"], 0, 0, 0, 'N', $impostoRegra["VRADICALIQFECP"], 0);
    
                        }
                        else {
                            $this->aprovacaoLancamentosFactory->deleteTrabimposNfTotImposto($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoRegra["CDIMPOSTO"]);
                            $this->aprovacaoLancamentosFactory->insereTrabimposNfTot($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoRegra["CDIMPOSTO"], 0);
                        }
                        $impostosInseridos[] = array("CDIMPOSTO" => $impostoRegra["CDIMPOSTO"]);
                    }
                    
                    $this->aprovacaoLancamentosFactory->deleteAliqTemp($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoRegra["CDIMPOSTO"]);
                    $this->aprovacaoLancamentosFactory->deleteInciTemp($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $impostoRegra["CDIMPOSTO"]);
                  
                }
            }

            $rateioAnterior = array_merge($rateioAnterior, $this->calculaRateioAnterior($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto));
            
            if($notafiscal->getIdentrsaidop() == 'S') {
                $impostosNota = $this->aprovacaoLancamentosFactory->getItemProImpos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $produto["NRSEQITPROD"]);
            }
            else {
                $impostosNota = $impostosInseridos;
            }
            
            $this->verificaImpostoObrigatorio($impostosObrigatorio, $impostosNota, $produto);
        }
        
        
        if($notafiscal->getIdentrsaidop() == 'E') {
            
            $somaImpostosTrabit = $this->aprovacaoLancamentosFactory->getSomaTrabitproimpos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
    
            foreach($somaImpostosTrabit as $somaImposto) {
               $this->aprovacaoLancamentosFactory->insereTrabimposNfTot($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $somaImposto["CDIMPOSTOIT"], $somaImposto["VRTOTAIMPO"]);
            }
            
            $totalNf = $notafiscal->getVrnotafisc();
            
            $valorNotaDepois = $this->proceduresNfFactory->procedureImpostoEntrada($notafiscal->getCdEmpresa(), $notafiscal->getCdinscesta(), $notafiscal->getCdtipooper(),
                                                       $fornCli, $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), 
                                                       $notafiscal->getDtentrsaid()->format('d/m/Y'), 
                                                       $this->environment->getCdOperador(), $nrlancestq);
                                                      
            $this->proceduresNfFactory->procedureGravaImposto($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        
        }
        
        $consContaRateio = $this->lancamentoFactory->getConsContaRateio($notafiscal->getIdentrsaidop(), $notafiscal->getCdtipooper())[0];
                                           
        if($notafiscal->getIdentrsaidop() == 'E') {
            $this->geraRateio($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getIdentrsaidop(),  $rateioAnterior, 'S', '');
        }
        else {
            $this->geraRateio($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getIdentrsaidop(),  $rateioAnterior, 'S', '');
        }
        
        $valorRetidoDepois = $this->aprovacaoLancamentosFactory->getValorRetidoPos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
        
        $consisteParametrizacaoFiscal = $this->aprovacaoLancamentosFactory->consisteParametrizacaoFiscal($notafiscal->getCdtipooper(), $notafiscal->getIdentrsaidop());
        if($consisteParametrizacaoFiscal) {
            $verificaCst = $this->aprovacaoLancamentosFactory->getCstImpostos($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());

            if(!empty($verificaCst)) {
                foreach($verificaCst as $cst) {
                    if($cst["CDSITUTRIBIT"] == 'XXX') {
                        throw new \Exception(self::MSG_ERROS_AUDITORIA["CST_IMP"]);
                    }
                    else if($cst["CDFISCOPERIT"] == 'XXX') {
                        throw new \Exception(self::MSG_ERROS_AUDITORIA["CFOP_IMP"]);
                    }
                }
            }
            else {
                $itenSemCstOuCfop = $this->aprovacaoLancamentosFactory->getCstItens($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf());
                
                if(!empty($itenSemCstOuCfop)) {
                    if(empty($itenSemCstOuCfop["CDSITUTRIBPR"])) {
                        throw new \Exception(self::MSG_ERROS_AUDITORIA["ERRO_CST"]);
                    }
                    else if (empty($itenSemCstOuCfop["CDFISOPEITEM"])){
                        throw new \Exception(self::MSG_ERROS_AUDITORIA["ERRO_CFOP"]);
                    }
                }
                
            }
            
        }
        
        if( ( ( $valorNotaAntes - $valorNotaDepois ) > 0.01 ) || ( ( $valorNotaDepois - $valorNotaAntes ) > 0.01 )
            || ( $valorRetidoAntes != $valorRetidoDepois ) ) {
                
            if($tipooperac->getIdutilvenc() != 'N') {
                throw new \Exception(self::MSG_ERROS_AUDITORIA["ALTERA_FIN"]);
            }
        }
        
        if($tipooperac->getCdtipooper() != '11') {
            $this->atualizaEstoque($notafiscal);
        }
    } 
    
    private function verificaImpostoObrigatorio($impostosObrigatorio, $impostosNota, $produto) {
        foreach($impostosObrigatorio as $impostoObrig) {
            $achou = false;
            $index = 0;
            
            foreach($impostosNota as &$imposto) {
                if($impostoObrig["CDIMPOSTO"] == $imposto["CDIMPOSTO"]) {
                    $achou = true;
                    unset($impostosObrigatorio[$index]);
                    break;
                }
                $index++;
            }

            if(!$achou) {
                throw new \Exception(self::MSG_ERROS_AUDITORIA["IMPOBR"]);
            }
            
        }
    }

    private function calculaRateioAnterior($cdfilial, $nrlanctonf, $produto) {
        $rateios = array();
        $rateio  = $this->aprovacaoLancamentosFactory->getRateioAnterior($cdfilial, $nrlanctonf, $produto["NRSEQITPROD"]);
        foreach($rateios as $rateio) {
            $rateio = array(
                "CDFILIAL"      => $cdfilial,
                "NRLANCTONF"    => $nrlanctonf,
                "NRSEQITPROD"   => $produto["NRSEQITPROD"],
                "CDFILICUST"    => $rateio["CDFILCCUST"],
                "CDCENTCUSNF"   => $rateio["CDCENTCUSNF"],
                "VRPERC"        => ($rateio["VRPERC"] > 0) ? $rateio["VRRATENF"] * 100 / $produto["VRCONTITEM"] : $rateio["VRPERC"]
            );
            $rateios[] = array_push($rateio);
        }

        return $rateios;
    }
        
    private function atualizaEstoque($notafiscal) {
        $this->proceduresNfFactory->procedureCustoLiquido($notafiscal->getCdfilial(), $notafiscal->getNrlanctonf(), $notafiscal->getVrprodutos());
        
        $filial = $notafiscal->getIdentrsaidop() == 'E' ? $notafiscal->getCdfilirece() : $notafiscal->getCdfiliemit();
        
        $paramfilial = $this->entityManager->getRepository('Geral\Entities\Paramfilial')->findOneBy(array('nrorg' => $notafiscal->getNrorg(), 'cdfilial' => $filial));
        
        if($paramfilial->getIdctrlestq() != 'N' && $paramfilial->getIdestonline() != 'N') {
            $lanctoestoq = $this->entityManager
                                 ->getRepository('Estoque\Entities\Lanctoestoq')
                                 ->findOneBy(array('nrorg' => $notafiscal->getNrorg(), 'idtplancto' => $notafiscal->getIdentrsaidop(), 
                                                   'cdfilial' => $notafiscal->getCdfilial(), 'nrlanctonf' => $notafiscal->getNrlanctonf()));
    
            if($lanctoestoq) {
                $dtFinal = $paramfilial->getDtprocessa();
                
                if(!$dtFinal) {
                    $dtFinal = date('d/m/Y');
                }
                else {
                    $dtFinal = $paramfilial->getDtprocessa()->format('d/m/Y');
                }
                
                $dtAtuEstoq = array('INITIAL' => $notafiscal->getDtentrsaid()->format('d/m/Y'), 'FINAL' => $dtFinal);
    
                $this->validacoesEstoqueService->atualizaEstoque($lanctoestoq->getCdfilial(), $lanctoestoq->getNrlancestq(), true, false, null, $dtAtuEstoq );
            }
            
        }
    }

    private function manifestaNaAprovacao($nrAcessoNfe, $cdEmpresa, $inscricao) {
        try {
            $paramManifesta = $this->aprovacaoLancamentosFactory->getParamManifesta();
            $idManif = null;
            if(isset($paramManifesta[0])) {
            
                if($paramManifesta[0]["IDCIENCIAEMISS"]  == '3') $idManif = '4';
                if($paramManifesta[0]["IDCONFIRMAEMISS"] == '3') $idManif = '1';
    
                $manifesta = false;
                $notaDestinada = $this->aprovacaoLancamentosFactory->getNotaDestinada($nrAcessoNfe);
                
                if(isset($notaDestinada[0])) {
                    if(($notaDestinada[0]["IDSTATUSMANIF"] == 0 
                    || $notaDestinada[0]["IDSTATUSMANIF"] == 4 
                    || $notaDestinada[0]["IDSTATUSMANIF"] == 5 ) && $notaDestinada[0]["IDSTATUSMANIF"] != $idManif) {
                        if($idManif) $manifesta = true;
                    }
                }
                else if($idManif) {
                    $manifesta = true;
                }

                if($manifesta === true) {
                    // Manifesta nota
                    $nrOrg   = $this->environment->getNrOrg();
                    $config  = $this->manifestacaoService->montaConfigManifestacao($cdEmpresa, $nrOrg, $inscricao, '', true);
                    $nota    = array("NRACESSONFE" => $nrAcessoNfe, "NRNOTAFISC" => "", "NRLANCTONF" => "");
                    $retorno = $this->manifestacaoService->manifestarNfe(array($nota), $config, ($idManif == 4) ? "210210" : "210200", '');
                    if(isset($retorno["ERRO"])) throw new \Exception($retorno["ERRO"][0]["MOTIVO"]);
                }
                
            }
        }
        catch(\Exception $e) {
        }
    }

    private function validaModDocFisc($cdmoddocfisc) {
        if ($cdmoddocfisc == null) {
            throw new \Exception('VALIDAMODDOCFISC');
        }
    }

    private function validaNotaFiscEstornada($dsnfestcanc) {
        if($dsnfestcanc == 'S') {
            throw new \Exception('VALIDANOTAFISCESTORNADA');
        }
    }

    private function validaIdErroNotaFisc($iderros) {
        if ($iderros[0] == '1') {
            throw new \Exception('VALIDAIDERRONOTAFISC');
        }
    }

    private function validaTipoDeConta($filial, $lancamento, $organizacao, $entrada_saida, $tipooperac){
        if ($entrada_saida === 'E') {
            $validaConta[0] = false;
            $tipoDeConta    = $this->aprovacaoLancamentosFactory->getTipoContaPagarProd($filial, $lancamento, $organizacao);
            $produtosNota   = $this->entityManager->getRepository('Geral\Entities\Itemproduto')->findBy(array('nrorg' => $organizacao, 'cdfilial' => $filial, 'nrlanctonf' => $lancamento));

			if (empty($tipoDeConta)) {
                $validaConta[0] = true;
            } else {
                foreach ($produtosNota as $produto) {
					foreach ($tipoDeConta as $tConta) {
						if ($tConta['CDPRODUTO'] == $produto->getCdproduto()) {
							$validaConta[0] = false;
							break;
						}
						$validaConta[0] = true;
					}
                }
            }

            $contapagar     = $this->entityManager->getRepository('Fiscal\Entities\Contapagar')->findOneBy(array('nrorg' => $organizacao, 'cdfilial' => $filial, 'nrlanctonf' => $lancamento));
            $validaConta[1] = !$contapagar instanceof Contapagar;

            $autoforne      = $this->aprovacaoLancamentosFactory->getAutoForne($filial, $lancamento, $organizacao);
            $validaConta[2] = empty($autoforne['CDTPCTPAGAR']);

            $tipoContaPagar = $this->aprovacaoLancamentosFactory->verificaTipoContaPagar($filial, $lancamento, $organizacao);
            $validaConta[3] = empty($tipoContaPagar['CDTPCTPAGAR']);

            if ($validaConta[0] && empty($tipooperac->getCdtpctpagar()) && !$validaConta[1] && $validaConta[2] && $validaConta[3]) {
                throw new \Exception('TIPO_CONTA_N_PARAM');
            }
        } elseif ($entrada_saida === 'S') {
            $contarecer = $this->entityManager->getRepository('Financeiro\Entities\Contareceber')->findOneBy(array('nrorg' => $organizacao, 'cdfilial' => $filial, 'nrlanctonf' => $lancamento));
            if($contarecer instanceof Contareceber){
                $contReceber = $this->aprovacaoLancamentosFactory->getContReceber($filial, $lancamento, $organizacao);

                if(empty($tipooperac->getCdtpctreceb()) && empty($contReceber)){
                    throw new \Exception('TIPO_CONTA_N_PARAM');
                }
            }
        }

        return true;
    }

    public function validaDataIntegracao($cdempresa, $identrsaiop, $tipooperac, $dtreferencia, $nrorg){
        $idcontaboper = $tipooperac->getIdcontaboper();

        $contopectbl = $this->entityManager->getRepository('Teknisa\Model\Contabilidade\Entities\Contopectbl')->findOneBy(array('nrorg' => $nrorg, 'cdempresa' => $cdempresa));

        if($contopectbl){
            $dtiniintcontdfe = $contopectbl->getDtiniintcontdfe();
            $dtfinintcontdfe = $contopectbl->getDtfinintcontdfe();
            $dtiniintcontdfs = $contopectbl->getDtiniintcontdfs();
            $dtfinintcontdfs = $contopectbl->getDtfinintcontdfs();
    
            if ($dtiniintcontdfe && $dtiniintcontdfs) {
    
                if($idcontaboper === 'S') {
    
                    if($identrsaiop === 'E') {
                        if ( !(((DateUtil::comparaDatas($dtiniintcontdfe, $dtreferencia, DateUtil::ANTES)) || (DateUtil::comparaDatas($dtiniintcontdfe, $dtreferencia, DateUtil::IGUAL))) &&
                             ((DateUtil::comparaDatas($dtfinintcontdfe, $dtreferencia, DateUtil::DEPOIS)) || (DateUtil::comparaDatas($dtfinintcontdfe, $dtreferencia, DateUtil::IGUAL))))
                           ) {
                            throw new \Exception('DTREFDFE');
                        }
                    }
                    else {
                        if ( !(((DateUtil::comparaDatas($dtiniintcontdfs, $dtreferencia, DateUtil::ANTES)) || (DateUtil::comparaDatas($dtiniintcontdfs, $dtreferencia, DateUtil::IGUAL))) &&
                            ((DateUtil::comparaDatas($dtfinintcontdfs, $dtreferencia, DateUtil::DEPOIS)) || (DateUtil::comparaDatas($dtfinintcontdfs, $dtreferencia, DateUtil::IGUAL))))
                        ) {
                            throw new \Exception('DTREFDFS');
                        }
                    }
    
                }
    
            }
        }

    }

    public function validaConsisteFiscal($cdfilial, $nrlanctonf, $nrnotafisc, $vrnotafisc, $dtemissao){
        $consisteFiscal = $this->aprovacaoLancamentosFactory->consisteFiscal($cdfilial, $nrlanctonf);
        $errosConsisteFiscal = array();
        if (is_array($consisteFiscal) && !empty($consisteFiscal)) {
            foreach ($consisteFiscal as $key => $erro) {
                $errosConsisteFiscal[$key]['NRNOTAFISC'] = $nrnotafisc;
                $errosConsisteFiscal[$key]['VRNOTAFISC'] = $vrnotafisc;
                if($dtemissao) $errosConsisteFiscal[$key]['DTEMISSAO'] = $dtemissao->format('d/m/Y');

                if (empty($erro['CDIMPOSTO']))
                    $errosConsisteFiscal[$key]['DSERRO'] = $erro['DSCONSISTENCIA'] . '(Produto:  ' . $erro['CDARVPROD'] . ')';
                else
                    $errosConsisteFiscal[$key]['DSERRO'] = $erro['DSCONSISTENCIA'] . ' (Produto:' . $erro['CDARVPROD'] . ' - Imposto: ' . $erro['CDIMPOSTO'] . ')';
            }
        }

        return $errosConsisteFiscal;
    }

    /**
     * Atualiza o status da nota fiscal para APROVADA
     */
    private function atualizaStatusNotaFiscalAprovada($notafiscal, $tipooperac)
    {
        $notafiscal->setIdstatus('A');
        $notafiscal->setIdbloqueado('S');
        $notafiscal->setCdoperapro($this->environment->getCdOperador());

        $dadosFilial = $this->validacoesEstoqueService->retornaDadosProcessFilial($notafiscal->getCdfilial(), $notafiscal->getNrorg());
        $notafiscal->setDtaprovacao(DateUtil::getDataDeString($dadosFilial['DTPROCESSA']));

        $notafiscal->setDtexportacao(null);
        $notafiscal->setIddocfinanc('N');
        $notafiscal->setIdexpnfep('N');
        $notafiscal->setIdexpnfemanaus('N');
        $notafiscal->setIdexpnfesanand('N');
        $notafiscal->setIdverifnfconf('N');

        if ($tipooperac->getIdgeravencnfap() === 'S') {
            $notafiscal->setIdbloqnffin('B');
            $notafiscal->setDtliberacaofin(null);
        } elseif ($tipooperac->getIdgeravencnfap() === 'N') {
            $notafiscal->setIdbloqnffin('L');
            $notafiscal->setDtliberacaofin(DateUtil::getDataAtual());
        }

        $notafiscal->setDtultatu(DateUtil::getDataAtual());
        $notafiscal->setCdoperultatu($this->environment->getCdOperador());

        $this->entityManager->flush();
    }

    /*---------    CÓDIGOS DE ERRO DE ACORDO COM A TELA CTB10200 ----------*/
    private static $array_erros = array(
        '001' => 'Total de crédito difere do total de débito.',
        '002' => 'Data fora do período contábil.',
        '003' => 'Partida %s - Conta contábil não cadastrada.',
        '004' => 'Partida %s - Histórico padrão não cadastrado.',
        '005' => 'Partida %s - Código comercial não cadastrado.',
        '006' => 'Partida %s - Valor da partida zerado.',
        '007' => 'Partida %s - Valor de rateio zerado.',
        '008' => 'Partida %s - Filial de rateio não cadastrada.',
        '009' => 'Partida %s - Centro de custo não cadastrado para filial informada.',
        '010' => 'Partida %s - Soma dos rateios difere do valor da partida.',
        '011' => 'Partida %s - Conta contábil sem código comercial informado.',
        '012' => 'Partida %s - Conta Contábil Gerencial sem rateio informado.',
        '013' => 'Partida %s - Centro de custo deve ser informado.',
        '014' => 'Produto %s sem conta contábil.',
        '015' => 'Lançamento fora do período contábil para o financeiro. (BCO, PAG e REC)',
        '016' => 'Lançamento fora do período contábil de documentos fiscais de entrada. (DFE)',
        '017' => 'Lançamento fora do período contábil para documentos fiscais de saída. (DFS)'
    );

    //Verifica se o lançamento possui erros de Contabilizaçã, caso possua
    //retorna true e abre a popUp, se nao retorna false;
    private function verificaErrosContabilizacao($cdempresa, $nrlancctb, $organizacao)
    {
        $erros = self::getDadosErros($cdempresa, $nrlancctb, $organizacao);
        if ($erros) {
            $dados_erros = array();

            foreach ($erros as $erro) {
                $err      = explode('|', $erro->getCderrotemp());
                $mensagem = self::$array_erros[$err[0]];
                if (strpos($mensagem, '%s')) {
                    $dserro = self::adicionaParametroMsgErro($mensagem, array($err[1]));
                } else {
                    $dserro = $mensagem;
                }
                $dados_erros[] = array('CDEMPRESA'     => $erro->getCdempresa(),
                                       'NRLANCCTBTEMP' => $erro->getNrlancctbtemp(),
                                       'NRSEQCTBTEMP'  => $erro->getNrseqctbtemp(),
                                       'CDERROTEMP'    => $erro->getCderrotemp(),
                                       'DSERRO'        => $dserro);

                return $dados_erros;
            }
        } else {
            return false;
        }
    }

    private function getDadosErros($cdempresa, $nrlancctb, $organizacao)
    {
        $erros = $this->entityManager->getRepository('Contabilidade\Entities\Erroctbtemp')->findOneBy(array('nrorg' => $organizacao, 'cdempresa' => $cdempresa, 'nrlancctbtemp' => $nrlancctb));

        return $erros;
    }

    private function adicionaParametroMsgErro($mensagem, $param = array())
    {
        foreach ($param as $replace) {
            $mensagem = substr_replace($mensagem, $replace, strpos($mensagem, '%s'), 2);
        }
        return $mensagem;
    }
    
    
    /*---------    Rateio ----------*/
    
    public function geraRateio($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut = 'N', $realizaManut = 'P'){
        $filiRate = '';
        $organizacao = $this->environment->getNrOrgTrab();
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            //Calcula rateio padrão
            if ($realizaManut <> 'S'){
                if($realizaManut == 'P'){
                    $rateionf = Rateionf::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                    foreach($rateionf as $rateio){
                        $rateio->delete();
                    }
                    $this->entityManager->flush();
                    $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                    $consContaRateio = $this->lancamentoFactory->getConsContaRateio($idEntrSaid, $notaFiscal->getCdtipooper())[0];
                    if($idEntrSaid == 'E'){
                        $this->geraRateioEntrada($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                    }else{ //($idEntrSaid == 'S' SAIDA
                        $this->geraRateioSaida($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                    }
                }
                else{// $realizaManut == 'S' Calcula rateio padrão
                    if($geraRatAut == 'N' && $realizaManut !== 'P'){ //aqui se rateio manual
                        $rateionf = Rateionf::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                        foreach($rateionf as $rateio){
                            $rateio->delete();
                        }
                        $this->entityManager->flush();
                        $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                        $consContaRateio = $this->lancamentoFactory->getConsContaRateio($idEntrSaid, $notaFiscal->getCdtipooper())[0];
                        if($idEntrSaid == 'E'){
                            $this->geraRateioEntrada($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                        }else{ //IDENTRSAID == 'S'
                            $this->geraRateioSaida($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                        }
                    }else{
                        $this->geraRateio($filial, $lancamento, $idEntrSaid, $rateioNF, 'N', 'P');
                        $this->calculaRateioPos($filial, $lancamento, $rateioNF, $organizacao);
                    }
                }
            }
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function calculaRateioPos($filial, $lancamento, $rateioNF , $organizacao){
        if(count($rateioNF) > 0){
            $itemproduto = $this->buscaProdutos($filial, $lancamento);
            foreach($itemproduto as $item){
                foreach($rateioNF as $rateioRow){
                    if($item['NRSEQITPROD'] == $rateioRow['NRSEQITPROD']){
                        $rateionf = Rateionf::findOneByCdfilialAndNrlanctonfAndNrorgAndNrseqitprod($filial, $lancamento, $organizacao, $item['NRSEQITPROD']);
                        $rateionf->delete();
                        $this->entityManager->flush();
                        $this->lancamentoFactory->InsertRatNovProd($rateioRow['CDFILIAL'], $rateioRow['NRLANCTONF'], $rateioRow['NRSEQITPROD'], $rateioRow['CDFILICCUST'], $rateioRow['CDCENTCUSNF'], $rateioRow['VRPERRAT'], $item['VRCONTITEM']);
                    }
                }
            }
            $totalContItem = $this->lancamentoFactory->getTotalCont($lancamento, $filial);
            foreach($totalContItem as $totalItem){
                $somaRateioItem = $this->lancamentoFactory->getSumRateio($lancamento, $filial, $totalItem['NRSEQITPROD'])[0]['VRRATEIONF'];
                if($somaRateioItem > 0){
                    $difCont = $totalItem['VRCONTITEM'] - $somaRateioItem;
                    if((abs($difCont) < 0.10) && (abs($difCont) > 0)){
                        $this->lancamentoFactory->updateRateionfRound($difCont, $filial, $lancamento, $totalItem['NRSEQITPROD']);
                    }
                }
            }
        }
    }
    
    public function buscaProdutos($filial, $lancamento) {
        $organizacao = $this->environment->getNrOrgTrab();
        $produtos = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        $arrayProdutos = $this->criaArrayProdutos($produtos);
        return $arrayProdutos;
    }
    
    
    public function geraRateioEntrada($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao){
        if($consContaRateio['IDCONSCTBRAT'] == 'S'){ //Considera tipo de conta para geração de rateio contabil
            $contapagar = $this->buscaVencimentos($filial, $lancamento, $idEntrSaid);
            if(is_null($contapagar))
                $tipoContNFEntr = $this->lancamentoFactory->getTipoContNF($filial, $lancamento, $idEntrSaid); 
            else
                $tipoContNFEntr = $contapagar->getCdtpctpagar();
            
            $tipoConta = $tipoContNFEntr['CDTPCTRECEB'];
            $itemproduto = $this->buscaProdutos($filial, $lancamento);
            foreach($itemproduto as $item){
                $ratCtEntr = $this->lancamentoFactory->getRatCt($tipoConta, $notaFiscal->getIdentrsaidop());
                foreach($ratCtEntr as $rateioRow){
                    if(!is_null($rateio['CDFILIAL']))
                        $filiRate = $rateioRow['CDFILIAL'];
                    else
                        $filiRate = $rateioRow['CDFILIRECE'];
                    
                    $existeRateio = Rateionf::findOneByCdfilialAndNrlanctonfAndNrseqitprodAndCdfiliccustAndCdcentcusnf($filial, $lancamento, $item->getNrseqitprod(), $filiRate, $ratCtEntr['CDCENTCUST']);
                    if(is_null($existeRateio))
                        $this->lancamentoFactory->insertRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioRow['VRPERATTPCTP']/100)), $rateioRow['CDCENTCUST'], $rateioRow['VRPERATTPCTP']);
                    else
                        $this->lancamentoFactory->updateRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioRow['VRPERATTPCTP']/100)), $rateioRow['CDCENTCUST'], $rateioRow['VRPERATTPCTP']);
                }
            }
        }
        else{ //$idconsctbrat['IDCONSCTBRAT'] <> S
            $this->lancamentoFactory->insertRateionfCompleto($filial, $lancamento, $notaFiscal->getCdfilirece(), $notaFiscal->getIdentrsaidop());
        }
    }
    
    public function geraRateioSaida($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao){
        if($consContaRateio['IDCONSCTBRAT'] == 'S'){
            $tipoContNFSaid = $this->lancamentoFactory->getTipoContNF($filial, $lancamento, $idEntrSaid);
            $tipoConta = $tipoContNFSaid['CDTPCTRECEB'];
            $itemproduto = $this->buscaProdutos($filial, $lancamento);
            foreach($itemproduto as $item){
                $ratCtSaid = $this->lancamentoFactory->getRatCt($tipoConta, $notaFiscal->getIdentrsaidop());
                foreach($ratCtSaid as $rateio){
                    if(!is_null($rateio['CDFILIAL']))
                        $filiRate = $rateioRow['CDFILIAL'];
                    else
                        $filiRate = $rateioRow['CDFILIRECE'];
                  
                    $existeRateio = Rateionf::findOneByCdfilialAndNrlanctonfAndNrseqitprodAndCdfiliccustAndCdcentcusnf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, $ratCtEntr['CDCENTCUST']);
                    if(is_null($existeRateio))
                        $this->lancamentoFactory->insertRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioRow['VRPERATTPCTP']/100)), $rateioRow['CDCENTCUST'], $rateioRow['VRPERATTPCTP']);
                    else
                        $this->lancamentoFactory->updateRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioRow['VRPERATTPCTP']/100)), $rateioRow['CDCENTCUST'], $rateioRow['VRPERATTPCTP']);
                }
            }
        }else{// $idconsctbrat['IDCONSCTBRAT'] <> 'S'
            $this->lancamentoFactory->insertRateionfCompleto($filial, $lancamento, $notaFiscal->getCdfilirece(), $notaFiscal->getIdentrsaidop());
        }
    }
    
       public function criaArrayProdutos(array $itemProdutos) {
        
        $organizacao    = $this->environment->getNrOrgTrab();
        $arrayProdutos  = array();
        if (!is_array($itemProdutos))
            return $arrayProdutos;

        /** @var Itemproduto $item */
        foreach ($itemProdutos as $key => $item) {
            if (!$item instanceof Itemproduto)
                throw new Exception('Produto não encontrado.');

            $produto    = Produto::findOneByCdprodutoAndNrorg($item->getCdproduto(), $organizacao);
            $prodfili   = Prodfili::findOneByCdfilialAndCdprodutoAndNrorg($item->getCdfilimovi(), $item->getCdproduto(), $organizacao);
            if($item->getCdservfili()) {
                $servico = $this->lancamentoFactory->buscaServico($item->getCdfilimovi(), $item->getCdservfili());
    
                if(isset($servico[0])) {
                    $arrayProdutos[$key]["NMPRODUTO"] = $servico[0]["NMSERVICO"];
                }
                else {
                    $arrayProdutos[$key]['NMPRODUTO'] = $produto->getNmproduto();
                }
            }
            else {
                $arrayProdutos[$key]['NMPRODUTO'] = $produto->getNmproduto();
            }
      
            $arrayProdutos[$key]['CDFILIAL']     = $item->getCdfilial();
            $arrayProdutos[$key]['NRLANCTONF']   = $item->getNrlanctonf();
            $arrayProdutos[$key]['CDPRODUTO']    = $item->getCdproduto();
            $arrayProdutos[$key]['NRSEQITPROD']  = $item->getNrseqitprod();
            $arrayProdutos[$key]['CDPRODEMIS']   = $item->getCdprodemis();
            $arrayProdutos[$key]['DSPRODEMIS']   = $item->getDsprodemis();
            $arrayProdutos[$key]['QTITEMPROD']   = $item->getQtitemprod();
            $arrayProdutos[$key]['VRUNITITEM']   = $item->getVrunititem();
            $arrayProdutos[$key]['VRTOTAITEM']   = round($item->getVrtotaitem(), 2);
            $arrayProdutos[$key]['VRDESCITEM']   = $item->getVrdescitem();
            $arrayProdutos[$key]['VRCONTITEM']   = $item->getVrcontitem();
            $arrayProdutos[$key]['CDSITUTRIBPR'] = $item->getCdsitutribpr();
            $arrayProdutos[$key]['CDFISOPEITEM'] = $item->getCdfisopeitem();
            $arrayProdutos[$key]['NRPEDPROD']    = $item->getNrpedprod();
            $arrayProdutos[$key]['NRSEQPEDPROD'] = $item->getNrseqpedprod();
            $arrayProdutos[$key]['QTVOLUITEM']   = $item->getQtvoluitem();
            $arrayProdutos[$key]['CDSERVFATU']   = $item->getCdservfatu();
            $arrayProdutos[$key]['CDSERVFILI']   = $item->getCdservfili();
            $arrayProdutos[$key]['CDALMOXARIFE'] = $item->getCdalmoxarife();
            $arrayProdutos[$key]['CDLOCALESTOQ'] = $item->getCdlocalestoq();
            $arrayProdutos[$key]['NRLOTEESTQ']   = $item->getNrloteestq();
            $arrayProdutos[$key]['NRSUBLOTE']    = $item->getNrsublote();
            $arrayProdutos[$key]['NRAUTOFORN']   = $item->getNrautoforn();
            $arrayProdutos[$key]['CDMARCA']      = $item->getCdmarca();

            $arrayProdutos[$key]['CDARVPROD']   = $produto->getCdarvprod();
            $arrayProdutos[$key]['SGUNIDADE']   = $produto->getSgunidade();
            $arrayProdutos[$key]['VRFATOCONV']  = $produto->getVrfatoconv();
            $arrayProdutos[$key]['CDPRODESTO']  = $produto->getCdprodesto();
            
            if(is_null($prodfili)) {
                $arrayProdutos[$key]['IDCNTRESTOQ']   = 'N';
            }
            else {
                $arrayProdutos[$key]['IDCNTRESTOQ'] = $prodfili->getIdcntrestoq();
            }
        }

        return $arrayProdutos;
    }
    
}